/*
 * Operationen ueber gemeine Brueche
 * Implementation der Funktionen aus bruch.h
 */

#include <stdio.h>
#include "bruch.h"

/* liest einen Bruch ein */
struct bruch bruchEingabe( void)
{
  struct bruch frac;
  printf( "Zaehler = "); scanf( "%d", &frac.Zaehler);
  printf( "Nenner = "); scanf( "%d", &frac.Nenner);
  return frac;
}

/* gibt einen Bruch aus */
void bruchAusgabe (struct bruch frac)
{
  if( isvalid( frac))
    if( frac.Nenner != 1)
      printf( "%d/%d", frac.Zaehler, frac.Nenner);
    else
      printf( "%d", frac.Zaehler);
  else
    printf( "NaN");
  return;
}

/* liefert zurck, ob ein Bruch eine gltige Zahl darstellt */
int isvalid( struct bruch frac)
{
  return frac.Nenner != 0;
}

/* liefert die Darstellung von NaN als Bruch */
struct bruch invalid( void)
{
  struct bruch frac;
  frac.Zaehler = frac.Nenner = 0;
  return frac;
}

/* krzt einen Bruch (Normalisierung) */
struct bruch kuerzen( struct bruch frac)
{ int sig, ggt;

/* Vorzeichen des Bruchs ermitteln */
  sig = signum( frac.Zaehler) * signum( frac.Nenner);
/* Komponenten nichtnegativ machen */
  frac.Zaehler = absi( frac.Zaehler);
  frac.Nenner = absi( frac.Nenner);
/* ggT ermitteln */
  ggt = ggT( frac.Zaehler,frac.Nenner);

  if( ggt > 0)
  {
    frac.Zaehler /= ggt;
    frac.Nenner /= ggt;
    frac.Zaehler *= sig;
  } 
  return frac;
}

/* Rckgabe des ggT zweier natuerlicher Zahlen */
unsigned int ggT( unsigned int r0, unsigned int r1)
{
  unsigned int v1,r2;
  while (r1 != 0)
  {
    v1 = r0 / r1; r2 = r0 % r1; r0 = r1; r1 = r2;
  }
  return r0;
}

/* liefert das Vorzeichen einer Ganzzahl */
int signum( int zahl)
{
  return ( zahl > 0) ? 1 : ( zahl < 0) ? -1 : 0;
}

/* liefert den Betrag einer Ganzzahl */
int absi (int zahl)
{
  return ( zahl < 0) ? -zahl : zahl;
}

/* liefert den Kehrwert eines Bruchs */
struct bruch inverse( struct bruch frac)
{
  struct bruch kehr;
  if( !isvalid( frac)) return invalid();
  kehr.Zaehler = frac.Nenner;
  kehr.Nenner = frac.Zaehler;
  return kehr;
}

/* liefert den Bruch mit umgekehrtem Vorzeichen */
struct bruch neg( struct bruch frac)
{ 
  frac.Zaehler = - frac.Zaehler;
  return frac;
}

/* liefert die Summe zweier Brche */
struct bruch add( struct bruch a, struct bruch b)
{
  struct bruch c;
  c.Zaehler = a.Zaehler * b.Nenner +
              b.Zaehler * a.Nenner;
  c.Nenner = b.Nenner * a.Nenner;
  return kuerzen( c);
}

/* liefert die Differenz zweier Brche */
struct bruch sub( struct bruch a, struct bruch b)
{
  struct bruch c;
  c = add( a, neg( b));
  return kuerzen( c);
}

/* liefert das Produkt zweier Brche */
struct bruch mult( struct bruch a, struct bruch b)
{
  struct bruch c;
  c.Zaehler = a.Zaehler * b.Zaehler;
  c.Nenner = a.Nenner * b.Nenner;
  return kuerzen( c);
}

/* liefert den Quotienten zweier Brche */
struct bruch div( struct bruch a, struct bruch b)
{
  return mult( a, inverse( b));
}
