/*
 * Wurzelberechnung
 * mittels Newton und Regula Falsi
 */
# include <stdio.h>
# include <math.h>

double eps;
double newton_wurzel( double, unsigned int, double);
double regulafalsi_wurzel
  ( double, unsigned int, double, double);
double mypow( double, unsigned int);

int main()
{
  unsigned int exp = 3;
  double rad = 27, start1 = 27, start2 = 1;

  for( eps=0.1; eps >= 0.00001; eps *= 0.1)
  {
    printf( "\nEPSILON = %e\n", eps);

    printf( "Newton( %f, %d)       = %f\n", rad, exp,
             newton_wurzel( rad, exp, start1));

    printf( "Regula_falsi( %f, %d) = %f\n", rad, exp,
        regulafalsi_wurzel( rad, exp, start1, start2));
  }

                               /* C-Standardfunktion */
  printf( "\nC: pow( %f, 1/%d)     = %f\n",
           rad, exp, pow( rad, 1 / (double)exp));

  return 0;
}

/*
 * Potenzfunktion
 * Berechnung von Potenzen mit natuerlichem Exponenten
 */
double mypow( double base, unsigned int exponent)
{
  unsigned int i; double power = 1;

  if( base == 0)
  {
    printf( "Fehler: Basis <> 0\n"); return 0;
  }
  for( i = 0; i < exponent; i++) power *= base;
  return power;
}

/*
 * Newtonverfahren
 * zur Wurzelberechnung
 * Parameter:  Radikant, Exponent, Startwert
 */
double newton_wurzel
  ( double a, unsigned int k, double x0)
{
  unsigned int n = 0; double x_n, x_n1, y;

  if( k <= 1 || a < 0)
  {
    printf( "Fehler: Wurzelexponent > 1!");
    printf( "Radikant >= 0 !\n");
    return -1;
  }

  x_n1 = x0;  y = mypow( x_n1, k);      /* Startwert */
  do
  {
    n++;
    x_n = x_n1;
    x_n1 = x_n - x_n * (1 - ( a / y)) / k; /* Newton */
    y = mypow( x_n1, k);
  } while( fabs( y - a) >= eps);
  printf( "n = %3d: ", n);     /* Iterationsschritte */

  return x_n1;
}

/*
 * Regula Falsi
 * zur Wurzelberechnung
 * Parameter:  Radikant, Exponent, Startwerte
 */
double regulafalsi_wurzel
  ( double a, unsigned int k, double x0, double x1)
{
  unsigned int n = 0;
  double x_n, x_n1, x_n2, y_n, y_n1, y_n2;

  if( k <= 1 || a < 0)
  {
    printf( "Fehler: Wurzelexponent > 1!");
    printf( "Radikant >= 0 !\n");
    return -1;
  }

  x_n1 = x0; x_n2 = x1;                /* Startwerte */
  y_n1 = mypow( x_n1, k) - a;
  y_n2 = mypow( x_n2, k) - a;

  if( y_n1 < 0 && y_n2 < 0 || y_n1 >= 0 && y_n2 >= 0)
  {
    printf( "Startwerte ungeeignet!"); return -1;
  }
 
  do
  {
    n++;
    
    x_n = x_n1; x_n1 = x_n2;
    y_n = y_n1; y_n1 = y_n2; 

 /* Regula falsi */
    x_n2 = x_n - y_n * ( x_n1 - x_n) / ( y_n1 - y_n);
    y_n2 = mypow( x_n2, k) - a;

    if( y_n1 < 0 && y_n2 < 0 || y_n1 >= 0 && y_n2 >= 0)
    {
      x_n1 = x_n; y_n1 = y_n;
    }
  } while( fabs( y_n2) >= eps);

  printf( "n = %3d: ", n);     /* Iterationsschritte */
  return x_n2 ;
}

/*
EPSILON = 1.000000e-01
n =   8: Newton( 27.000000, 3)           = 3.000073
n = 211: Regula_falsi( 27.000000, 3)     = 2.996372

EPSILON = 1.000000e-02
n =   8: Newton( 27.000000, 3)           = 3.000073
n = 280: Regula_falsi( 27.000000, 3)     = 2.999641

EPSILON = 1.000000e-03
n =   9: Newton( 27.000000, 3)           = 3.000000
n = 348: Regula_falsi( 27.000000, 3)     = 2.999963

EPSILON = 1.000000e-04
n =   9: Newton( 27.000000, 3)           = 3.000000
n = 417: Regula_falsi( 27.000000, 3)     = 2.999996

EPSILON = 1.000000e-05
n =   9: Newton( 27.000000, 3)           = 3.000000
n = 486: Regula_falsi( 27.000000, 3)     = 3.000000

C-Standardfunktion: pow( 27.000000, 1/3) = 3.000000
*/